@@ -25,12 +25,12 @@ use warnings;
 use Carp;
 
 use Exporter;
-use Convert::ASN1 qw(:all);
+use Convert::ASN1;
 use MIME::Base64;
 
 our @EXPORT  = qw();
 our @ISA     = qw(Exporter);
-our $VERSION = 1.1;
+our $VERSION = 1.2;
 
 my %oids = (
     '2.5.4.6'                       => 'countryName',
@@ -185,15 +185,20 @@ sub _convert_extensionRequest {
 
     my $extensionRequest = shift;
     my $parser = _init('extensionRequest');
-    my $decoded = $parser->decode($extensionRequest) or confess $parser->error, ".. looks like damaged input";
+    my $decoded = $parser->decode($extensionRequest) or return [];
     foreach my $entry (@{$decoded}) {        
         if (defined $oids{ $entry->{'extnID'}}) {
+            my $parser = _init($oids{ $entry->{'extnID'}});
+            if(!$parser) {
+                $entry = undef;
+                next;
+            }
             $entry->{'extnID'} = $oids{ $entry->{'extnID'} };
-            my $parser = _init($entry->{'extnID'}) or confess "parser error: ", $entry->{'extnID'}, " needs entry in ASN.1 definition!";
             $entry->{'extnValue'} = $parser->decode($entry->{'extnValue'}) or confess $parser->error, ".. looks like damaged input";
             $entry->{'extnValue'} = $self->_mapExtensions($entry->{'extnID'}, $entry->{'extnValue'});
         }
     }
+    @{$decoded} = grep { defined } @{$decoded};
     return $decoded;
 }
 
@@ -228,14 +233,20 @@ sub _mapExtensions {
 
 sub _convert_rdn {
     my $self = shift;
-
     my $typeandvalue = shift;
     my %hash;
+    my @array;
     foreach my $entry ( @{$typeandvalue} ) {
         if (defined $oids{ $entry->[0]->{'type'}}) {
-            $hash{ $oids{ $entry->[0]->{'type'} } } = $entry->[0]->{'value'};
+            if (defined $hash{ $oids{ $entry->[0]->{'type'} } }) {
+                push $hash{ $oids{ $entry->[0]->{'type'} } }, (values $entry->[0]->{'value'})[0];
+            }
+            else {
+                $hash{ $oids{ $entry->[0]->{'type'} } } = [(values $entry->[0]->{'value'})[0]];
+            }
         }
     }
+
     return \%hash;
 }
 
@@ -374,29 +385,42 @@ ASN1
 
 sub commonName {
     my $self = shift;
-    return $self->{'certificationRequestInfo'}->{'subject'}->{'commonName'}
-        {'utf8String'} || '';
+    return $self->{'certificationRequestInfo'}->{'subject'}->{'commonName'}->[0] || '';
 }
 
 sub organizationalUnitName {
     my $self = shift;
-    return $self->{'certificationRequestInfo'}->{'subject'}->{'organizationalUnitName'}->{'utf8String'} || '';
+    return $self->{'certificationRequestInfo'}->{'subject'}->{'organizationalUnitName'}->[0] || '';
+}
+
+sub organizationName {
+    my $self = shift;
+    return $self->{'certificationRequestInfo'}->{'subject'}->{'organizationName'}->[0] || '';
 }
 
 sub emailAddress {
     my $self = shift;
-    return $self->{'certificationRequestInfo'}->{'subject'}->{'emailAddress'}
-        {'ia5String'} || '';
+    return $self->{'certificationRequestInfo'}->{'subject'}->{'emailAddress'}->[0] || '';
 }
 
 sub stateOrProvinceName {
     my $self = shift;
-    return $self->{'certificationRequestInfo'}->{'subject'}->{'stateOrProvinceName'}->{'utf8String'} || '';
+    return $self->{'certificationRequestInfo'}->{'subject'}->{'stateOrProvinceName'}->[0] || '';
 }
 
 sub countryName {
     my $self = shift;
-    return $self->{'certificationRequestInfo'}->{'subject'}->{'countryName'}->{'printableString'} || '';
+    return $self->{'certificationRequestInfo'}->{'subject'}->{'countryName'}->[0] || '';
+}
+
+#this is an alternative function which allows to deal with multivalued subjects by returning an array instead of a single value
+sub domainComponent {
+    my $self = shift;
+    my @return;
+    foreach (@{$self->{'certificationRequestInfo'}->{'subject'}->{'domainComponent'}}) {
+        push @return, $_;
+    }
+    return @return;
 }
 
 sub version {
@@ -448,6 +472,25 @@ sub certificateTemplate {
     return $template; 
 }
 
+sub extensionValue {
+    my $self = shift;
+    my $extensionName = shift;
+    my %attributes = attributes($self);
+    my $value;
+    my @space = @{$attributes{'extensionRequest'}};
+    foreach my $entry (@space) {
+        if ($entry->{'extnID'} eq $extensionName) {
+            $value = $entry->{'extnValue'};
+            # reduce the hash items to the scalar value            
+            while (ref $value eq 'HASH') {
+                my @keys = keys %{$value};
+                $value = $value->{ shift @keys } ;
+            }            
+        }
+    }
+    return $value; 
+}
+
 1;
 
 __END__
@@ -492,6 +535,10 @@ Returns the common name as stored in the request.
 
 Returns the organizational unit name.
 
+=head2 organizationName
+
+Returns the organization name.
+
 =head2 emailAddress
 
 Returns the email address.
